# encoding: utf-8

import unittest
import datetime
from timingsutil import Throttle
import logging_helper
logging = logging_helper.setup_logging()


class TestConfiguration(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_throttle(self):

        t = Throttle(2)

        self.assertEqual(t.interval, 0.5)

        for _ in range(5):
            current = datetime.datetime.now()
            t.wait()
            self.assertGreaterEqual(datetime.datetime.now(), current - datetime.timedelta(milliseconds=500))


if __name__ == u'__main__':
    unittest.main()



# encoding: utf-8

import unittest
import datetime
from timingsutil import TimeIncrementer
import logging_helper
logging = logging_helper.setup_logging()


class TestTimeIncrementer(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_time_offseter_with_defaults(self):
        tinc = TimeIncrementer()
        PERIOD = TimeIncrementer.DEFAULT_PERIOD
        self.assertEqual(tinc._period.seconds, PERIOD)
        self.assertEqual(tinc.start, tinc.current)
        self.assertEqual((tinc.next() - tinc.start).seconds, PERIOD)
        self.assertEqual((tinc.current - tinc.start).seconds, PERIOD)
        self.assertEqual((tinc.next() - tinc.start).seconds, PERIOD * 2)
        self.assertEqual((tinc.current - tinc.start).seconds, PERIOD * 2)

    def test_non_default_period(self):
        PERIOD = 600
        tinc = TimeIncrementer(period=PERIOD)
        self.assertEqual(tinc._period.seconds, PERIOD)
        self.assertEqual(tinc.start, tinc.current)
        self.assertEqual((tinc.next() - tinc.start).seconds, PERIOD)
        self.assertEqual((tinc.current - tinc.start).seconds, PERIOD)
        self.assertEqual((tinc.next() - tinc.start).seconds, PERIOD * 2)
        self.assertEqual((tinc.current - tinc.start).seconds, PERIOD * 2)

    def test_timedelta_period(self):
        PERIOD = datetime.timedelta(minutes=8)
        tinc = TimeIncrementer(period=PERIOD)
        self.assertEqual(tinc._period, PERIOD)
        self.assertEqual(tinc.start, tinc.current)
        self.assertEqual((tinc.next() - tinc.start), PERIOD)
        self.assertEqual((tinc.current - tinc.start), PERIOD)
        self.assertEqual((tinc.next() - tinc.start), PERIOD * 2)
        self.assertEqual((tinc.current - tinc.start), PERIOD * 2)

    def test_string_start_without_format(self):
        self.assertRaises(ValueError, TimeIncrementer, start=u"10:00")

    def test_string_start_with_bad_format(self):
        self.assertRaises(ValueError,
                          TimeIncrementer,
                          start=u"10:00",
                          fmt=u"%H")

    def test_string_start_with_good_format(self):
        tinc = TimeIncrementer(start=u"10:00",
                               fmt=u"%H:%M")
        PERIOD = TimeIncrementer.DEFAULT_PERIOD
        self.assertEqual(tinc._period.seconds, PERIOD)
        self.assertEqual(tinc.start, tinc.current)
        self.assertEqual(tinc.start, u"10:00")
        self.assertEqual(tinc.next(), u"10:05")
        self.assertEqual(tinc.current, u"10:05")
        self.assertEqual(tinc.next(), u"10:10")
        self.assertEqual(tinc.current, u"10:10")


if __name__ == u'__main__':
    unittest.main()



# encoding: utf-8

from datetime import datetime as _dt

TZ_ADJUST = (_dt.now() - _dt.utcnow()).seconds

ONE_SECOND = 1
FIVE_SECONDS = 5 * ONE_SECOND
TEN_SECONDS = 10 * ONE_SECOND
FIFTEEN_SECONDS = 15 * ONE_SECOND
TWENTY_SECONDS = 20 * ONE_SECOND
TWENTY_FIVE_SECONDS = 25 * ONE_SECOND
THIRTY_SECONDS = 30 * ONE_SECOND
THIRTY_FIVE_SECONDS = 35 * ONE_SECOND
FORTY_FIVE_SECONDS = 45 * ONE_SECOND
FIFTY_SECONDS = 50 * ONE_SECOND
FIFTY_FIVE_SECONDS = 55 * ONE_SECOND
SIXTY_SECONDS = 60 * ONE_SECOND

ONE_MINUTE = SIXTY_SECONDS
FIVE_MINUTES = 5 * ONE_MINUTE
TEN_MINUTES = 2 * FIVE_MINUTES
FIFTEEN_MINUTES = 3 * FIVE_MINUTES
THIRTY_MINUTES = 2 * FIFTEEN_MINUTES
FORTY_FIVE_MINUTES = 3 * FIFTEEN_MINUTES

ONE_HOUR = 60 * ONE_MINUTE
TWO_HOURS = 2 * ONE_HOUR
THREE_HOURS = 3 * ONE_HOUR
FOUR_HOURS = 2 * TWO_HOURS
FIVE_HOURS = 5 * ONE_HOUR
SIX_HOURS = 6 * ONE_HOUR
SEVEN_HOURS = 7 * ONE_HOUR
EIGHT_HOURS = 8 * ONE_HOUR
NINE_HOURS = 9 * ONE_HOUR
TEN_HOURS = 10 * ONE_HOUR

TWELVE_HOURS = 12 * ONE_HOUR
EIGHTEEN_HOURS = 18 * ONE_HOUR
TWENTY_FOUR_HOURS = 24 * ONE_HOUR
THIRTY_SIX_HOURS = 36 * ONE_HOUR
FORTY_EIGHT_HOURS = 48 * ONE_HOUR

ONE_DAY = TWENTY_FOUR_HOURS
TWO_DAYS = 2 * ONE_DAY

ONE_WEEK = 7 * ONE_DAY
TWO_WEEKS = 2 * ONE_WEEK

# Constants below here are all based on multiples of one week so may not be exact

ONE_MONTH = 4 * ONE_WEEK
THREE_MONTHS = 13 * ONE_WEEK
SIX_MONTHS = 26 * ONE_WEEK
NINE_MONTHS = 39 * ONE_WEEK

ONE_YEAR = 52 * ONE_WEEK



# encoding: utf-8

# Variables for setup (these must be string only!)
__module_name__ = u'timingsutil'
__description__ = u'A collection of timing utilities.'

__version__ = u'1.5.4'

__author__ = u'Hywel Thomas'
__authorshort__ = u'HT'
__authoremail__ = u'hywel.thomas@mac.com'

__license__ = u'MIT'

__githost__ = u'bitbucket.org'
__gituser__ = u'daycoder'
__gitrepo__ = u'timingsutil.git'


# Additional variables
__copyright__ = u'Copyright (C) 2016 {author}'.format(author=__author__)

__url__ = u'https://{host}/{user}/{repo}'.format(host=__githost__,
                                                 user=__gituser__,
                                                 repo=__gitrepo__)
__downloadurl__ = u'{url}/get/{version}.tar'.format(url=__url__,
                                                    version=__version__)



# encoding: utf-8

import logging_helper
from .timeout import Timeout
logging = logging_helper.setup_logging()


class Throttle(object):

    def __init__(self,
                 rate=None,
                 interval=None):

        """
        Perpetually restarting timer
        Example use is to limit the number of requests per second to a service

        :param rate: allowed events per second
        :param interval: time in seconds before next event is permitted
        """

        if rate:
            try:
                self.interval = 1.0 / rate

            except ZeroDivisionError:
                self.interval = 0

        else:
            self.interval = interval

        # logging.debug(u'Throttle interval:{interval}'
        #               .format(interval=interval))

        self.timeout = Timeout(seconds=self.interval)

    def wait(self,
             message=None):
        self.timeout.wait(message=message)
        self.timeout.restart()



# encoding: utf-8

import logging_helper
from .timeout import TimersBase

logging = logging_helper.setup_logging()


class Stopwatch(TimersBase):

    def __init__(self):
        self.reset()

    def reset(self):
        self.__stop_time = None
        self.__laps = []
        self.__start_time = self._now

    def stop(self):
        if self.__stop_time is None:
            self.__stop_time = self._now

            return self.glance

        else:
            return self.__stop_time

    def lap(self):
        lap_end_time = self._now
        lap_start_time = (self.__start_time
                          if not self.__laps
                          else self.__laps[-1][u'lap_end_time'])
        self.__laps.append({
            u'lap_start_time': lap_start_time,
            u'lap_end_time':   lap_end_time,
            u'lap_time':       lap_end_time - lap_start_time
        })

        return self.__laps[-1][u'lap_time']

    @property
    def lap_times(self):
        return self.__laps

    @property
    def glance(self):
        if self.__stop_time:
            return self.__stop_time - self.__start_time

        else:
            return self._now - self.__start_time



# encoding: utf-8

import time
import unittest
import datetime
import timeit
from timingsutil.timeout import Timeout
import logging_helper
logging = logging_helper.setup_logging()


class TestConfiguration(unittest.TestCase):

    TOLERANCE = 0.005

    def now(self):
        return timeit.default_timer()

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_timeout(self):

        timeout = Timeout(-1)
        self.assertLessEqual(timeout.seconds_remaining, 0)
        self.assertEqual(timeout.time_remaining, datetime.timedelta(seconds=0))
        self.assertTrue(timeout.expired)

        timeout = Timeout(0)
        self.assertEqual(timeout.seconds_remaining, 0)
        self.assertEqual(timeout.time_remaining, datetime.timedelta(seconds=0))
        self.assertTrue(timeout.expired)

        timeout = Timeout(5)
        self.assertGreaterEqual(timeout.seconds_remaining, 4)
        self.assertGreaterEqual(timeout.time_remaining, datetime.timedelta(seconds=4))
        self.assertFalse(timeout.expired)
        time.sleep(2)
        self.assertGreaterEqual(timeout.seconds_remaining, 2)
        self.assertGreaterEqual(timeout.time_remaining, datetime.timedelta(seconds=2))
        self.assertFalse(timeout.expired)
        time.sleep(3 + self.TOLERANCE)
        self.assertTrue(timeout.expired, timeout.seconds_remaining)

    def wait_check(self,
                   wait):
        start_time = self.now()
        Timeout(wait).wait()
        time_after_wait = self.now() + self.TOLERANCE
        self.assertGreaterEqual(time_after_wait,
                                start_time + wait,
                                start_time + wait - time_after_wait)

    def test_5s_timeout_wait(self):
        self.wait_check(5)

    def test_half_second_timeout_wait(self):
        self.wait_check(0.5)

    def test_timeout_not_expired(self):
        wait = 5
        start_time = self.now()
        iteration = 0
        timeout = Timeout(wait)

        while not timeout.expired:
            iteration += 1
            time.sleep(1)
            self.assertGreaterEqual(timeout.elapsed_time + self.TOLERANCE, iteration)
            self.assertGreaterEqual(timeout.seconds_remaining, 0)

        self.assertGreaterEqual(self.now(), start_time + wait)

    # TODO: Add tests for interruptable


if __name__ == u'__main__':
    unittest.main()
